package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.exception.CannotDeleteException
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import dev.fyloz.colorrecipesexplorer.model.validation.NullOrNotBlank
import javax.persistence.*
import javax.validation.constraints.NotBlank
import javax.validation.constraints.NotNull

private const val COMPANY_ID_NULL_MESSAGE = "Un identifiant est requis"
private const val COMPANY_NAME_NULL_MESSAGE = "Un nom est requis"

@Entity
@Table(name = "company")
data class Company(
  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  override val id: Long?,

  @Column(unique = true)
  override val name: String
) : NamedModel


open class CompanySaveDto(
  @field:NotBlank(message = COMPANY_NAME_NULL_MESSAGE)
  val name: String
) : EntityDto<Company> {
    override fun toEntity(): Company = Company(null, name)
}


open class CompanyUpdateDto(
  @field:NotNull(message = COMPANY_ID_NULL_MESSAGE)
  val id: Long,

  @field:NullOrNotBlank(message = COMPANY_NAME_NULL_MESSAGE)
  val name: String?
) : EntityDto<Company> {
    override fun toEntity(): Company = Company(id, name ?: "")
}

// ==== DSL ====
fun company(
  id: Long? = null,
  name: String = "name",
  op: Company.() -> Unit = {}
) = Company(id, name).apply(op)

fun companySaveDto(
  name: String = "name",
  op: CompanySaveDto.() -> Unit = {}
) = CompanySaveDto(name).apply(op)

fun companyUpdateDto(
  id: Long = 0L,
  name: String? = "name",
  op: CompanyUpdateDto.() -> Unit = {}
) = CompanyUpdateDto(id, name).apply(op)

// ==== Exceptions ====
private const val COMPANY_NOT_FOUND_EXCEPTION_TITLE = "Company not found"
private const val COMPANY_ALREADY_EXISTS_EXCEPTION_TITLE = "Company already exists"
private const val COMPANY_CANNOT_DELETE_EXCEPTION_TITLE = "Cannot delete company"
private const val COMPANY_EXCEPTION_ERROR_CODE = "company"

fun companyIdNotFoundException(id: Long) =
  NotFoundException(
    COMPANY_EXCEPTION_ERROR_CODE,
    COMPANY_NOT_FOUND_EXCEPTION_TITLE,
    "A company with the id $id could not be found",
    id
  )

fun companyNameNotFoundException(name: String) =
  NotFoundException(
    COMPANY_EXCEPTION_ERROR_CODE,
    COMPANY_NOT_FOUND_EXCEPTION_TITLE,
    "A company with the name $name could not be found",
    name,
    "name"
  )

fun companyIdAlreadyExistsException(id: Long) =
  AlreadyExistsException(
    COMPANY_EXCEPTION_ERROR_CODE,
    COMPANY_ALREADY_EXISTS_EXCEPTION_TITLE,
    "A company with the id $id already exists",
    id
  )

fun companyNameAlreadyExistsException(name: String) =
  AlreadyExistsException(
    COMPANY_EXCEPTION_ERROR_CODE,
    COMPANY_ALREADY_EXISTS_EXCEPTION_TITLE,
    "A company with the name $name already exists",
    name,
    "name"
  )

fun cannotDeleteCompany(company: Company) =
  CannotDeleteException(
    COMPANY_EXCEPTION_ERROR_CODE,
    COMPANY_CANNOT_DELETE_EXCEPTION_TITLE,
    "Cannot delete the company ${company.name} because one or more recipes depends on it"
  )
