package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeEditRecipes
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeRemoveRecipes
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewRecipes
import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.service.MixService
import dev.fyloz.colorrecipesexplorer.service.RecipeImageService
import dev.fyloz.colorrecipesexplorer.service.RecipeService
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import java.net.URI
import javax.validation.Valid


private const val RECIPE_CONTROLLER_PATH = "api/recipe"
private const val MIX_CONTROLLER_PATH = "api/recipe/mix"

@RestController
@RequestMapping(RECIPE_CONTROLLER_PATH)
@PreAuthorizeViewRecipes
class RecipeController(private val recipeService: RecipeService) {
    @GetMapping
    fun getAll() =
      ok(recipeService.getAll())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
      ok(recipeService.getById(id))

    @PostMapping
    @PreAuthorizeEditRecipes
    fun save(@Valid @RequestBody recipe: RecipeSaveDto) =
      created<Recipe>(RECIPE_CONTROLLER_PATH) {
          recipeService.save(recipe)
      }

    @PutMapping
    @PreAuthorizeEditRecipes
    fun update(@Valid @RequestBody recipe: RecipeUpdateDto) =
      noContent {
          recipeService.update(recipe)
      }

    @PutMapping("public")
    @PreAuthorize("hasAuthority('EDIT_RECIPES_PUBLIC_DATA')")
    fun updatePublicData(@Valid @RequestBody publicDataDto: RecipePublicDataDto) =
      noContent {
          recipeService.updatePublicData(publicDataDto)
      }

    @DeleteMapping("{id}")
    @PreAuthorizeRemoveRecipes
    fun deleteById(@PathVariable id: Long) =
      noContent {
          recipeService.deleteById(id)
      }
}

@RestController
@RequestMapping(RECIPE_CONTROLLER_PATH)
@PreAuthorizeViewRecipes
class RecipeImageController(val recipeImageService: RecipeImageService) {
    @GetMapping("{recipeId}/image")
    fun getAllIdsForRecipe(@PathVariable recipeId: Long) =
      ok(recipeImageService.getAllIdsForRecipe(recipeId))

    @GetMapping("{recipeId}/image/{id}", produces = [MediaType.IMAGE_JPEG_VALUE, MediaType.IMAGE_PNG_VALUE])
    fun getById(@PathVariable recipeId: Long, @PathVariable id: Long) =
      ok(recipeImageService.getByIdForRecipe(id, recipeId))

    @PostMapping("{recipeId}/image", consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorizeEditRecipes
    fun save(@PathVariable recipeId: Long, image: MultipartFile): ResponseEntity<Void> {
        val id = recipeImageService.save(image, recipeId)
        return ResponseEntity.created(URI.create("/$RECIPE_CONTROLLER_PATH/$recipeId/image/$id")).build()
    }

    @DeleteMapping("{recipeId}/image/{id}")
    @PreAuthorizeRemoveRecipes
    fun delete(@PathVariable recipeId: Long, @PathVariable id: Long) =
      noContent {
          recipeImageService.delete(id, recipeId)
      }
}

@RestController
@RequestMapping(MIX_CONTROLLER_PATH)
@PreAuthorizeViewRecipes
class MixController(private val mixService: MixService) {
    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
      ok(mixService.getById(id))

    @PostMapping
    @PreAuthorizeEditRecipes
    fun save(@Valid @RequestBody mix: MixSaveDto) =
      created<Mix>(MIX_CONTROLLER_PATH) {
          mixService.save(mix)
      }

    @PutMapping
    @PreAuthorizeEditRecipes
    fun update(@Valid @RequestBody mix: MixUpdateDto) =
      noContent {
          mixService.update(mix)
      }

    @DeleteMapping("{id}")
    @PreAuthorizeRemoveRecipes
    fun deleteById(@PathVariable id: Long) =
      noContent {
          mixService.deleteById(id)
      }
}
